Dear Don,

> Any comments??  Anyone?
Yes! Prepare yourself for a long answer.
Without ft_onidle I get a CPU load of 3 to 4 percent! This is what you
want.

Your main goal is to get CPU down when waiting for a key or mouse event.
This message tells
you how. To be complete I also tell something about getting CPU load
down during other CLipper processing.


Solutions:
----------------------------------------------------------------------
1. FT_onidle( {|| FT_IAMIDLE()} )

2. Do while ( nKey := inkey() ) = 0
     FT_IAMIDLE()
   Enddo

3. DEFINE BACKGROUNDEVENT ACTION FT_IAMIDLE() INTERVAL .04
   Do while ( nKey := inkey() ) = 0
   Enddo


Results:
----------------------------------------------------------------------
    CPU-load
1.  100%            EEK?! 
2.  3-7%
3.  3-7%


Sourcecode 1
----------------------------------------------------------------------
Function main()
  Local nKey
  Local nCount := 0
  @ 0,0 clear
  FT_onidle( {||devpos( 3,0 ), devout( time() ), devout( ++nCount) } )
  Do while ( nKey := inkey() ) = 0
  Enddo

  Return .T.


  Conclusion 1
  ----------------------------------------------------------------------
  The codeblock passed by FT_OnIdle() is called afew times and then
hardly any more.
  Hence: FT_OnIdle() doesn't work the way you expect. It consumes CPU 


Sourcecode 2
----------------------------------------------------------------------
   Do while ( nKey := inkey() ) = 0
     FT_IAMIDLE()
   Enddo

  
  Conclusion 2
  ----------------------------------------------------------------------
  Works good. My only guess is that FT_IAMIDLE is called a lot more than 
  needed to get CPU-load down.


Sourcecode 3
----------------------------------------------------------------------
   DEFINE BACKGROUNDEVENT ACTION FT_IAMIDLE() NAME "TIMESLICE" INTERVAL
.04
   Do while ( nKey := inkey() ) = 0
   Enddo
   RELEASE BACKGROUNDEVENT "TIMESLICE"

  Conclusion 3
  ----------------------------------------------------------------------
  Works good, with interval values of .04 or less. 


More research results
----------------------------------------------------------------------
The use of inkey(<num>) with a value higher than <num> increases
CPU load drasticly in solution 2. A background event solves the problem.
DEFINE BACKGROUNDEVENT ACTION FT_IAMIDLE() NAME "TIMESLICE" INTERVAL .04

I was interested to know how often FT_IAMIDLE was called in solution 2
and 3.
The results were interesting.

Function main()
  Local nKey
  Local nCount := 0
  Local nSeconds := 5
  @ 0,0 clear
  SET CURSOR OFF

  *----------------------
  @ 0,0 say "Solution 2"
  *----------------------
  * A background event is used to break out of do while loop
  * inkey(num) may NOT be used it increases CPU-load by itself already.

  DEFINE BACKGROUNDEVENT ACTION BREAK(.T.) NAME "BREAK" interval 5
  
  @ 7,0 say time()
  begin sequence
     Do while inkey() = 0
        ft_IAMIDLE()
        nCount++
     Enddo
  end sequence
  @ 8,0 say time()
  @ 10,0 say nCount
  @ 11,0 say int( nCount/nSeconds )

  RELEASE EVENT "BREAK"

  *----------------------
  @ 0,40 say "Solution 3"
  *----------------------
  @ 7,40 say time()
  DEFINE BACKGROUNDEVENT ACTION (FT_IAMIDLE(), nCount++) INTERVAL .04
  nCount := 0    // Reset !!
  inkey( nSeconds )
  @ 8,40 say time()
  @ 10,40 say nCount
  @ 11,40 say int( nCount/nSeconds )

  @ 18,0
  Return .T.


The background events are called 18 times each second. It seems
background events are linked to the clock: 18 hits per second.
I didn't test FT_ONTICK()

In solution 2 Inkey() gives a key at least 141 times each second!!!
When there are a lot of things you do between while...end, you probably
are going to need solution 2. 
Otherwise both solution do fine.

CPU-decrease during execution
*------------------------------
You could give away a few timeslices during processing Clipper code.
You can do this by inserting FT_IAMIDLE() on strategic spots in your
source. 
If you use background events, you have to slow down the execution of the
events.
Use: BACKGROUND EVENTS INTERVAL <nSeconds.nHundreds>. Only the main
background event function, which checks if it may
process another event, will be called. 
If you don't, you will experience a decrease in performance of your
tbrowse, indexing or for..next loop.

Concerning your RAT_EVENT
*------------------------
> I have moosed my apps and use SuperLib's Rat_Event() function in place
> of inkey() and was wondering if Ft_Idle() in Rat_Event() while ...
> enddo loop would be as good as FT_ONIDLE( { || FT_IAMIDLE() } )?

If RAT_event does the same as FT_ONIDLE, then you have to use
FT_IAMIDLE.
Is FT_Idle a typo in your message?


Well that is that. It kept me busy for two hours.
Any Christmas Gift can be send to Eindhoven.<g>

Please let me know how you go.

Kind regards,
Robert Lamping

Eindhoven,
The Netherlands



PersonaL:  http://www.stack.nl/~lamping
BGEVENTS:  http://www.stack.nl/~lamping/bgevents.htm      (FREEWARE)


 
> On Fri, 19 Dec 1997 02:45:40 GMT, null@iag.net (Phil Barnett) wrote
> (possibly edited):
> 
> >You could download the freeware library nanfor, and add the following
> >line to your top module.
> >
> >FT_ONIDLE( FT_IAMIDLE() )
> 
> **  Should that be : FT_ONIDLE( { || FT_IAMIDLE() } ) ?
> >
> >which automatically detects idle states in Clipper and returns the
> >time slices back to the O/S. Other changes are not needed.
> >
> >Some methods work better than others. You'll need to expirement.
> 
> Phil,
> 
> I have moosed my apps and use SuperLib's Rat_Event() function in place
> of inkey() and was wondering if Ft_Idle() in Rat_Event() while ...
> enddo loop would be as good as FT_ONIDLE( { || FT_IAMIDLE() } )?
> 
> Any comments??  Anyone?
> 
> Don
>
